@@ -1,8 +1,11 @@
 #ifndef NMAP_LUA_NSOCK_H
 #define NMAP_LUA_NSOCK_H
 
-#include "nse_main.h"
+extern "C" {
+  #include "lua.h"
+}
 
 LUALIB_API int luaopen_nsock (lua_State *);
 
 #endif
+
